a_molecule = [
				# a list of models
		[		# a list of chains
			["A",[
				[1081,"","ASN",[
					[[" O  ",""],
					[1.0,22.87," O"],
					[0.287,20.822,40.742]],
					[[" C  ",""],
					[1.0,22.17," C"],
					[0.303,20.8649,39.5379]],
					[[" ND2",""],
					[1.0,29.17," N"],
					[1.363,16.5869,37.908]],
					[[" OD1",""],
					[1.0,32.810001373291," O"],
					[2.877,18.049,38.597]],
					[[" CG ",""],
					[1.0,27.4," C"],
					[1.711,17.676,38.529]],
					[[" CB ",""],
					[1.0,24.58," C"],
					[0.547,18.528,39.102]],
					[[" CA ",""],
					[1.0,23.18," C"],
					[0.657,19.99,38.7]],
					[[" N  ",""],
					[1.0,22.87," N"],
					[0.531,20.0909,37.2929]]]
				],
				[1082,"","GLN",[
					[[" O  ",""],
					[1.0,19.223," O"],
					[0.419,24.068,39.868]],
					[[" C  ",""],
					[1.0,20.54," C"],
					[1.431,23.549,40.3401]],
					[[" NE2",""],
					[1.0,14.77," N"],
					[5.637,22.1359,39.458]],
					[[" OE1",""],
					[1.0,25.2299," O"],
					[5.2859,23.3729,41.34]],
					[[" CD ",""],
					[1.0,25.7," C"],
					[5.08,23.136,40.01]],
					[[" CG ",""],
					[1.0,19.98," C"],
					[4.176,24.007,39.253]],
					[[" CB ",""],
					[1.0,21.63," C"],
					[3.038,23.178,38.604]],
				  	[[" CA ",""],
					[1.0,20.56," C"],
					[2.118,22.47,39.593]],
				  	[[" N  ",""],
					[1.0,21.99," N"],
					[1.156,21.65,38.894]]]
				]
			    ]	# end of chain A content
			]	# end of chain A
		]		# end of list of chains
	]			# end of list of models

# Return a python_mol
#
#
#
def jiggled_mol(reference_mol, current_mol, traj_frac):

	def jiggle_random():
		import random
		return (random.random() - 0.5)

	def jiggled_pos(ref_pos, current_pos):
		if (traj_frac < 0):	# magic value
			# make a starting set of coords
			return map(lambda x1: 
				jiggle_random() * 1.0 + x1,
				ref_pos)
		else:
			q = 1 - traj_frac 
			return map(lambda x, x_ref:
				traj_frac * x_ref + q * x + 
				0.4 * q * jiggle_random(),
				current_pos, ref_pos)

	def jiggled_atom(ref_atom, current_atom):
		ref_pos = ref_atom[2]
		cur_pos = current_atom[2]
		ret = [ref_atom[0], ref_atom[1], jiggled_pos(ref_pos, cur_pos)]
		return ret

	def jiggled_residue(ref_res, cur_res):
		ret = [ref_res[0], ref_res[1], ref_res[2], map(jiggled_atom, ref_res[3], cur_res[3])]
		return ret

	def jiggled_chain(ref_chain, cur_chain):
		ret = [ref_chain[0],
			       map(jiggled_residue, 
				   ref_chain[1],
				   cur_chain[1])]
		return ret

	def jiggled_model(ref_model, cur_model):
		ret = map(jiggled_chain, ref_model, cur_model)
		return ret

        j_mod = map (jiggled_model, reference_mol, current_mol)
	return j_mod


#
def disrupt(reference_mol, biggness):

	ret = jiggled_mol(reference_mol, reference_mol, -1)
	return ret

#
max_count = 5000.0

mol_no = add_molecule(a_molecule, "test molecule")
if (not mol_no == -1):
	set_rotation_centre(*centre_of_mass(mol_no))
	for count in range(int(max_count) + 1):
		current_mol = disrupt(a_molecule,0.8)
		new_mol = jiggled_mol(a_molecule, current_mol, 
			count / max_count)
		#print "cycle ", count, count/max_count, max_count
		clear_and_update_molecule(mol_no, new_mol)
		# gtk stuff
		if gtk.events_pending():
			gtk.main_iteration(False)




import types
import gi
gi.require_version("Gtk", "4.0")
from gi.repository import Gtk

import coot
import coot_utils
import coot_gui
import res_spec_utils
import redefine_functions as rdf

# consider for coot_utils
def residue_spec_to_string(spec):
    # print("DEBUG:: in residue_spec_to_string() with spec", spec)
    if len(spec) == 4:
        return (spec[1] + str(spec[2]) + spec[3])
    else:
        return (spec[0] + str(spec[1]) + spec[2])

# find bad things in the structure - rama, C-beta, rotamer, atom clashes baddies
#
def validation_outliers_dialog(imol, imol_map):

    dialog_vbox = False
    window = False
    missing_sidechains_checkbutton = False
    cg_torsion_diff_checkbutton = False
    poor_density_checkbutton =  False

    def find_rama_baddies():
        rs = coot.all_molecule_ramachandran_score_py(imol)
        scored_residues = rs[5]
        interesting = filter(lambda item: item[2] < 0.02, scored_residues)

        # remove phi-psi and res-names from the return value
        munged = map(lambda item: [item[1], item[2]], interesting)
        return ["Ramachandran Improbables", munged]

    # a list of atom specs
    def find_chiral_volume_baddies():
        r = coot.chiral_volume_errors_py(imol)
        if not isinstance(r, list):
            return []
        else:
            return r

    def make_window_title(n):
        return "Coot Interesting/Outliers/Problems: " + str(n)

    def find_c_beta_baddies():
        try:
            c_beta_deviations
        except:
            return []
        return c_beta_deviations(imol)

    def find_em_ringer_baddies():
        try:
            CG_spin_research
        except:
            return []
        if not isinstance(scored_residues, list):
            return []
        else:
            scored_residues = CG_spin_research(imol, imol_map)
            interesting = filter(lambda item: item[1] > 30 or item[1] < -30,
                                 scored_residues)
            # return a list of residue specs - not what is expected?
            return interesting

    def rotamer_score_residues(imol):

        residues = coot_utils.all_residues_sans_water(imol)
        ret = map(lambda residue_spec: [residue_spec,
                                        coot.rotamer_score(imol,
                                                      res_spec_utils.residue_spec_to_chain_id(residue_spec),
                                                      res_spec_utils.residue_spec_to_res_no(residue_spec),
                                                      coot_utils.residue_spec_to_ins_code(residue_spec),
                                                      "")],
                  residues)
        return ret

    def filter_rotamer_baddies(baddies):

        het_groups_in_mol = coot.het_group_residues_py(imol)

        ret = []

        for baddie in baddies:
            spec, score = baddie
            res_name = coot.residue_name(imol,
                                    res_spec_utils.residue_spec_to_chain_id(spec),
                                    res_spec_utils.residue_spec_to_res_no(spec),
                                    coot_utils.residue_spec_to_ins_code(spec))

            if res_name in ["ALA", "GLY", "UNK", "HOH"]:
                pass
            elif (score == 0.0 and not ok_to_have_missing_sidechain_buttons_qm()):
                pass
            else:
                # if spec is a het-group then no rotamers for that (return False)
                is_het = any(map(lambda item: residue_specs_match_qm(item, spec),
                                 het_groups_in_mol))
                if is_het:
                    pass
                else:
                    if res_name == "LEU":
                        if score < 1.1:
                            ret += [baddie]
                    else:
                        if score < 0.021:
                            ret += [baddie]
        return ret

    def molecule_atom_overlap_baddies():
        l = coot.molecule_atom_overlaps_py(imol)
        if isinstance(l, list):
            return l
        else:
            info_dialog(l)
            return []

    def filter_molecule_atom_overlap_baddies(mao_baddies):
        baddie_limit = 2.0  # more than this is marked as a baddie, was 2.2. Is 2.0 good?
        return filter(lambda mao_item: mao_item['overlap-volume'] > baddie_limit, mao_baddies)

    def non_pro_cis_peptide_baddies():  # do the filter here - just for consistency
        cis_peps = coot.cis_peptides_py(imol)
        return filter(lambda peptide: "PRO" != coot_utils.residue_spec_to_residue_name(imol, peptide[1]), cis_peps)

    def twisted_trans_peptide_baddies():
        return coot.twisted_trans_peptides_py(imol)

    def destroy_buttons_with_label(label_fragment_txt, dialog_vbox):
        current_buttons = dialog_vbox.get_children()
        for button in current_buttons:
            label = button.get_label()
            if label_fragment_txt in label:
                button.destroy()

    def regenerate_button_fn(*args):

        def cb_func(button, callback):
            callback("some-arg")

        if dialog_vbox:
            old_buttons = dialog_vbox.get_children()
            for butt in old_buttons:
                butt.destroy()
            buttons = make_buttons()
            for button_parts in buttons:
                # print("button parts 0", button_parts[0])
                # print("button parts 1", button_parts[1])
                button = Gtk.Button(button_parts[0])
                button.connect("clicked", cb_func, button_parts[1])
                dialog_vbox.append(button)
                button.show()
            if window:
                window.set_title(make_window_title(len(buttons)))
            # validation_outliers_dialog(imol, imol_map)

    def ok_to_do_density_correlations_qm():
        if poor_density_checkbutton:
            return poor_density_checkbutton.get_active()
        else:
            return True

    def ok_to_have_missing_sidechain_buttons_qm():
        if missing_sidechains_checkbutton:
            return missing_sidechains_checkbutton.get_active()
        else:
            return True

    def ok_to_do_CG_torsion_diffs_qm():
        try:
            CG_spin_search
        except:
            return False
        return cg_torsion_diff_checkbutton.get_active()

    def go_to_atom_in_molecule(imol, atom_spec):
        coot.set_go_to_atom_molecule(imol)
        rdf.set_go_to_atom_from_atom_spec(atom_spec)

    def go_to_residue_in_molecule(imol, res_spec):
        coot.set_go_to_atom_molecule(imol)
        rdf.set_go_to_atom_from_res_spec(res_spec)

    def generator(res_spec):
        func = lambda res_spec_c=res_spec : go_to_residue_in_molecule(imol, res_spec_c)
        def action(arg):
            func(res_spec)
        return action

    def generator_for_atom_spec(atom_spec):
        func = lambda atom_spec_c=atom_spec : go_to_atom_in_molecule(imol, atom_spec_c)
        def action(arg):
            func(atom_spec)
        return action

    def make_buttons():

        frb = find_rama_baddies()
        fcbb = find_c_beta_baddies()
        filtered_mao_baddies = filter_molecule_atom_overlap_baddies(molecule_atom_overlap_baddies())
        residue_correlations = [] if not ok_to_do_density_correlations_qm() \
                               else coot.map_to_model_correlation_per_residue_py(imol,
                                                                                 coot_utils.all_residues_sans_water(imol),
                                                                                 0,
                                                                                 imol_map)

        def get_rama_prob(val):
            return val[1]

        # rama
        baddies = filter(lambda baddie: baddie[1] < 0.002, frb[1])
        # baddies.sort(key=get_rama_prob)
        sorted_filtered_rama_baddies = baddies

        # c-beta
        c_beta_baddies = filter(lambda baddie: baddie[1][0][1] > 0.25, fcbb)
        def get_c_beta_score(val):
            return val[1][0][1]
        # c_beta_baddies.sort(key=get_c_beta_score, reverse=True)
        sorted_filtered_c_beta_baddies = c_beta_baddies

        # Density correlations
        #
        density_baddies = filter(lambda baddie: baddie[1] < 0.8, residue_correlations)

        # CG Torsion
        #
        cg_torsion_baddies = find_em_ringer_baddies()

        # Rotamers
        #
        rotamer_baddies = rotamer_score_residues(imol)
        filtered_rotamer_baddies = filter_rotamer_baddies(rotamer_baddies)

        def get_rotamer_score(val):
            if val[1] == 0.0:
                # score things with score 0.0 (meaning missing sidechain)
                # as if they are better than low probability outliers
                # give high score in sorting
                return 10
            else:
                return val[1]
        sorted_filtered_rotamer_baddies = filtered_rotamer_baddies.sort(key=get_rotamer_score)

        rama_buttons = []
        for baddie in sorted_filtered_rama_baddies:
            spec, rama_prob = baddie
            score_string = '{:6.2f} %'.format(100 * rama_prob)
            button_label = "Ramachandran Outlier " + \
                           res_spec_utils.residue_spec_to_chain_id(spec) + \
                           " " + \
                           str(coot_utils.residue_spec_to_res_no(spec)) + \
                           coot_utils.residue_spec_to_ins_code(spec) + \
                           " " + \
                           coot_utils.residue_spec_to_residue_name(imol, spec) + \
                           " " + \
                           score_string
            rama_buttons.append([button_label, generator(spec)])

        c_beta_buttons = []
        for baddie in sorted_filtered_c_beta_baddies:
            spec = baddie[0]
            score = baddie[1][0][1]  # only the first score
            score_string = '{:6.2f}'.format(score)
            button_label = "C-beta deviant " + \
                           residue_spec_to_string(spec) + \
                           " " + \
                           coot_utils.residue_spec_to_residue_name(imol, spec) + \
                           " " + \
                           score_string + u'\u212B'.encode('utf-8')
            c_beta_buttons.append([button_label, generator(residue_spec)])

        non_pro_cis_peptide_buttons = []
        for baddie in non_pro_cis_peptide_baddies():
            spec_1 = baddie[0]
            spec_2 = baddie[1]
            omega = baddie[2]
            button_label = "Non-PRO cis-peptide " + \
                           residue_spec_to_string(spec_1) + \
                           " - " + \
                           residue_spec_to_string(spec_2)
            non_pro_cis_peptide_buttons.append([button_label, generator(spec_1)])

        twisted_trans_peptide_buttons = []
        for baddie in twisted_trans_peptide_baddies():
            spec_1 = baddie[0]
            spec_2 = baddie[1]
            omega = baddie[2]
            button_label = "Twisted trans-peptide " + \
                           residue_spec_to_string(spec_1) + \
                           " - " + \
                           residue_spec_to_string(spec_2)
            twisted_trans_peptide_buttons.append([button_label, generator(spec_1)])

        rota_buttons = []
        for baddie in filtered_rotamer_baddies:
            spec, score = baddie

            # Paul is not sure that he likes a score of
            # 0.0 meaning "Missing sidechain"
            # we have lost some information on the way.
            # 20200511-PE Yeah, like the fact that the residue was RNA!
            #
            score_string = '{:6.2f} %'.format(score)
            ms_string = "Missing Sidechain" if score == 0.0 else "Rotamer Outlier"
            rot_name = coot.get_rotamer_name_py(imol,
                                        res_spec_utils.residue_spec_to_chain_id(spec),
                                        res_spec_utils.residue_spec_to_res_no(spec),
                                        coot_utils.residue_spec_to_ins_code(spec))
            button_label = ms_string + " " + \
                           residue_spec_to_string(spec) + \
                           " " + \
                           coot_utils.residue_spec_to_residue_name(imol, spec) + \
                            " "
            button_label += rot_name if isinstance(rot_name, str) else " "
            button_label += "" if score == 0.0 else score_string
            rota_buttons.append([button_label, generator(spec)])

        density_baddies_buttons = []
        for baddie in density_baddies:
            spec, score = baddie
            button_label = "Poor Density Fit " + \
                           residue_spec_to_string(spec) + \
                           " " + \
                           '{:5.2f}'.format(score)
            density_baddies_buttons.append([button_label, generator(spec)])

        cg_torsion_buttons = []
        for baddie in cg_torsion_baddies:
            spec, score = baddie
            button_label = "CG Torsion Diff " + \
                           residue_spec_to_string(spec) + \
                           " " + \
                           '{:5.2f}'.format(score)
            cg_torsion_buttons.append([button_label, generator(spec)])

        chiral_volume_buttons = []
        for baddie_atom_spec in find_chiral_volume_baddies():
            atom_spec = baddie_atom_spec
            if len(atom_spec) == 6:
                atom_spec = atom_spec[1:]
            button_label = "Chiral Volume Error " + coot_utils.atom_spec_to_string(atom_spec)
            func = generator_for_atom_spec(atom_spec)
            chiral_volume_buttons.append([button_label, func])

        atom_overlap_buttons = []
        for baddie in filtered_mao_baddies:
            atom_spec_1 = baddie['atom-1-spec']
            atom_spec_2 = baddie['atom-2-spec']
            overlap = baddie['overlap-volume']
            button_label = "Atom Overlap " + \
                           coot_utils.atom_spec_to_string(atom_spec_1) + \
                           " on " + \
                           coot_utils.atom_spec_to_string(atom_spec_2) + \
                           " OV: " + \
                           '{:5.2f}'.format(overlap)
            atom_overlap_buttons.append([button_label, generator_for_atom_spec(atom_spec_1)])

        buttons =  chiral_volume_buttons + \
                   rama_buttons + \
                   rota_buttons + \
                   non_pro_cis_peptide_buttons + \
                   twisted_trans_peptide_buttons + \
                   density_baddies_buttons + \
                   c_beta_buttons + \
                   cg_torsion_buttons + \
                   atom_overlap_buttons
        return buttons

    # main line

    buttons = make_buttons()

    dialog_vbox, window = coot_gui.dialog_box_of_buttons(make_window_title(len(buttons)),
                                                [350, 400], buttons, " Close ")

    window_bits = window.get_children()
    vbox_outer = window_bits[0]
    control_button_vbox_1 = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
    missing_sidechains_checkbutton_local = Gtk.CheckButton("Missing Sidechains")
    poor_density_checkbutton_local = Gtk.CheckButton("Poor Density Fit")
    cg_torsion_diff_checkbutton_local = Gtk.CheckButton("CG Torsion Diff.")
    regenerate_button_local = Gtk.Button("Update")

    missing_sidechains_checkbutton = missing_sidechains_checkbutton_local
    poor_density_checkbutton = poor_density_checkbutton_local
    cg_torsion_diff_checkbutton = cg_torsion_diff_checkbutton_local

    missing_sidechains_checkbutton.set_active(True)
    poor_density_checkbutton.set_active(True)
    cg_torsion_diff_checkbutton.set_active(True)
    control_button_vbox_1.append(missing_sidechains_checkbutton)
    control_button_vbox_1.append(poor_density_checkbutton)

    vbox_outer.append(regenerate_button_local)
    vbox_outer.append(control_button_vbox_1)

    regenerate_button_local.connect("clicked", regenerate_button_fn)

    def missing_sidechains_checkbutton_toggled(widget):
        state = widget.get_active()
        if not state:
            # i.e. no buttons with "Missing Sidechain"
            # Mmmh, really?!?
            destroy_buttons_with_label("Missing Sidechain", dialog_vbox)

    missing_sidechains_checkbutton.connect("toggled",
                                           missing_sidechains_checkbutton_toggled)

    def poor_density_checkbutton_toggled(widget):
        state = widget.get_active()
        if not state:
            destroy_buttons_with_label("Poor Density", dialog_vbox)

    poor_density_checkbutton.connect("toggled",
                                     poor_density_checkbutton_toggled)

    # 20190102-PE depends on the version of coot that we are using
    #
    have_cg_spin = False
    try:
        CG_spin_search
        have_cg_spin = True
    except:
        pass
    if have_cg_spin:
        control_button_vbox_1.append(cg_torsion_diff_checkbutton)
        cg_torsion_diff_checkbutton.show()

        def cg_torsion_diff_checkbutton_toggled(widget):
            state = widget.get_active()
            if not state:
                destroy_buttons_with_label("CG TOrsion", dialog_vbox)
        cg_torsion_diff_checkbutton.connect("toggled",
                                            cg_torsion_diff_checkbutton_toggled)

    control_button_vbox_1.show()
    missing_sidechains_checkbutton.show()
    poor_density_checkbutton.show()
    regenerate_button_local.show()




# unittest_extensions.py
# Copyright 2007, 2008, 2009 by The University of York
# Copyright 2008, 2009 by Bernhard Lohkamp
# Copyright 2007 by Paul Emsley
# Copyright 2007 by The University of Oxford
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
home_dir = os.getenv('HOME')
if (not home_dir and os.name == 'nt'):
        home_dir = os.getenv('COOT_HOME')

project_dir = os.path.join(home_dir, "Projects", "coot", "python-tests")
unittest_filename = os.path.join(project_dir, "coot_unittest.py")
begin_filename = os.path.join(project_dir, "begin.py")


def exec_file(filename):
    if (os.path.isfile(unittest_filename) and
        (filename == "coot_unittest.py")):
        execfile(unittest_filename, globals())
    elif (os.path.isfile(begin_filename) and
          (filename == "begin.py")):
        execfile(begin_filename, globals())
    else:
        unittest_dir = "python-tests"
        unittest_file = filename
        # assume we started coot in bin:
        full_dir = os.path.normpath(os.path.join("..", unittest_dir))
        if (os.path.isdir(full_dir)):
            full_name = os.path.join(unittest_dir, unittest_file)
            if (os.path.isfile(full_name)):
                execfile(full_name, globals())
            else:
                print "BL INFO:: could not find %s in ../%s" %(unittest_file, unittest_dir)
        else:
            print "BL INFO:: could not find directory ../%s" %unittest_dir
            # search, starting from HOME
            if (os.name == 'nt'):
                home = os.getenv("COOT_HOME")
            else:
                home = os.getenv("HOME")
            # now we assume its in ~/Projects/coot
            full_dir = os.path.normpath(os.path.join(home, "Projects", "coot", unittest_dir))
            if (os.path.isdir(full_dir)):
                full_name = os.path.join(full_dir, unittest_file)
                if (os.path.isfile(full_name)):
                    execfile(full_name, globals())
                else:
                    print "BL INFO:: could not find %s in %s" %(unittest_file, full_dir)
            else:
                print "BL INFO:: searching $HOME..."
                for root, dirs, files in os.walk(home):
                    #print "BL DEBUG:: root, dirs", root, dirs
                    if unittest_dir in dirs:
                        full_name = os.path.join(root, unittest_dir, unittest_file)
                        if (os.path.isfile(full_name)):
                            execfile(full_name, globals())
                            break
                        else:
                            print "BL INFO:: could not find %s in %s/%s" %(unittest_file, root, unittest_dir)
                            print "BL INFO:: continue searching..."

if (have_coot_python):
    if coot_python.main_menubar():

        menu = coot_gui.coot_menubar_menu("Unittesting")


        def run_all_tests_func():
            exec_file("coot_unittest.py")

        coot_gui.add_simple_coot_menu_menuitem(menu, "Run All Unittest Tests",
                                      lambda func: run_all_tests_func())


        def run_test_set_gui():
            exec_file("begin.py")
            #print "BL DEBUG:: test list", test_list
            #for tmp in test_list:
            #    print "BL DEBUG:: test str", str(tmp).split(".")[1].rstrip("\'>")

            # make buttons list:
            buttons = []
            for test in test_list:
                buttons.append([str(test).split(".")[1].rstrip("\'>"), "run_test_set(" + str(test_list.index(test)) + ")"])

            #run_test_set(7)

            coot_gui.dialog_box_of_buttons("Coot Unittests Sets", [200, 320], buttons,
                                  "  Close  ")

            

        coot_gui.add_simple_coot_menu_menuitem(menu, "Run A Unittest Set...",
                                      lambda func: run_test_set_gui())

        def run_one_test_gui():
            exec_file("begin.py")
            test_list = list_of_all_tests()

            # make buttons list:
            buttons = []
            prev_test_class = "PdbMtzTestFunction"
            for test in test_list:
              if (not prev_test_class in test[2]):
                # new set, let's introduce
                #print "BL DEBUG:: test id of new set", test[2]
                buttons.append(["HSep"])
              buttons.append([test[1], "run_one_test(" + str(test_list.index(test)) + ")"])
              prev_test_class = test[2].split(".")[1]
            
            coot_gui.dialog_box_of_buttons("Coot Unittests", [400, 500], buttons,
                                  "  Close  ")
            

        coot_gui.add_simple_coot_menu_menuitem(menu, "Run One Unittest Test...",
                                      lambda func: run_one_test_gui())





#load the rcrane module
import rcrane

def import_rcrane_wrapper():
   
   #create the menu entry in the Extensions menu
   extMenu = coot_gui.coot_menubar_menu("Calculate")
   rcraneLaunch = gtk.MenuItem("RCrane launch")
   rcraneLaunch.connect("activate", lambda x: rcrane.createRCraneMenuWithCheck())
   rcraneLaunch.show()
   extMenu.insert(rcraneLaunch, len(extMenu.get_children())-1)

   #destroy any variables that we've created, since the variables will be visible from the
   # Coot Python scripting window
   del extMenu
   del rcraneLaunch

   #check to see if the user has an old version of RCrane installed
   #if they do, stop it from running
   def checkForOldRCraneVersions():
       from rcrane.settings import settingsFilename
       rcraneSettingsFilename = settingsFilename()

       #try to open the RCrane settings file
       try:
           rcraneSettings = open(rcraneSettingsFilename, 'r')
       except IOError:
           #if the file doesn't exist, then the user doesn't have an old version of RCrane installed
           return False

       for curline in rcraneSettings:
           if curline[0:18] == "run_python_script(":
               rcraneSettings.close()
               break
       else:
           #the rcrane.py file exists, but it doesn't have a coot.run_python_script( line,
           #so the user doesn't have an old version of RCrane installed
           rcraneSettings.close()
           return False

       #if we reach here, then the user has an old version of RCrane installed
       from shutil import move
       from os.path import dirname, join

       #move the user's rcrane.py to rcrane.py.bak, since it's no longer necessary
       try:
           move(rcraneSettingsFilename, join(dirname(rcraneSettingsFilename), "rcrane.py.bak"))
       except IOError:
           print("Cannot move ~/.coot-preferences/rcrane.py to ~/.coot-preferences/rcrane.py.bak")
           return False

       #let the user know what we just did
       noticeDialog = gtk.MessageDialog(type = gtk.MESSAGE_WARNING, buttons = gtk.BUTTONS_OK)
       noticeDialog.set_title("Old version of RCrane detected")
       noticeDialog.set_markup("An old version of RCrane was detected.  Coot now includes "
           + "RCrane 1.1, which may be accessed via Extensions -> RCrane launch.  To prevent the "
           + "old version of RCrane from running, your "
           + "~/.coot-preferences/rcrane.py file was moved to ~/.coot-preferences/rcrane.py.bak.")
       noticeDialog.run()
       noticeDialog.destroy()

       return True


   checkForOldRCraneVersions()
   del checkForOldRCraneVersions #destroy the function as well, since we don't need it after it's been run

# do this now if we load stuff from guile-gtk

# print '===== in rcrane loader: use_gui_qm is ', use_gui_qm

if (use_gui_qm != 2):
    import_rcrane_wrapper()



#!/usr/bin/env python
"""List the maximum length of a bond that Coot will draw as connected."""

# Copyright 2010, 2011, 2012 Kevin Keating
# 
# Licensed under the Educational Community License, Version 2.0 (the
# "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
# 
# http://www.osedu.org/licenses/ECL-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.

from coot import coot_version

LONG_BOND_DIST_CUTOFF = 2.16

#Coot 0.6.2 and 0.7 have different distance cut-offs for deciding whether or not to draw a bond
if coot_version()[0:3] == "0.6":
    BOND_DIST_CUTOFF  = 1.64 #value for the Coot's 0.6.2 branch
else:
    BOND_DIST_CUTOFF  = 1.71 #value for the Coot trunk (i.e. 0.7 and newer)

BOND_LIST = [(" P  ", " OP1", LONG_BOND_DIST_CUTOFF),   #the P-OP1 and P-OP2 bonds must be the first two elements of this list
             (" P  ", " OP2", LONG_BOND_DIST_CUTOFF),   #since we use BOND_LIST[0:2] as the bond list for the last nucleotide
             (" P  ", " O5'", LONG_BOND_DIST_CUTOFF),
             (" C1'", " C2'", BOND_DIST_CUTOFF),
             (" C2'", " C3'", BOND_DIST_CUTOFF),
             (" C3'", " C4'", BOND_DIST_CUTOFF),
             (" C3'", " O3'", BOND_DIST_CUTOFF),
             (" C4'", " C5'", BOND_DIST_CUTOFF),
             (" C4'", " O4'", BOND_DIST_CUTOFF),
             (" O4'", " C1'", BOND_DIST_CUTOFF),
             (" C5'", " O5'", BOND_DIST_CUTOFF),
             (" C2'", " O2'", BOND_DIST_CUTOFF)]

# Note that when using BOND_DIST_CUTOFF, bonds greater than or equal to the distance are not drawn.
# When using LONG_BOND_DIST_CUTOFF, bond greater than the distance are not drawn,
# but bonds equal to the distance *are* drawn.
# For now, we ignore that distinction, since getting a bond length of *exactly* 1.64 (or 1.71) should be fairly uncommon

# Also note that the P-O3' bond uses the BOND_DIST_CUTOFF, not LONG_BOND_DIST_CUTOFF
# This is presumably a bug/oversight in Coot due to the fact that it's an inter-residue bond.


#a list of all bonds (excluding hydrogens) expected for backbone and nucleosides
BOND_LIST_FULL = {"A": [], "G": [], "C": [], "U": [], "backbone": []}

for (atom1, atom2, bondLength) in BOND_LIST:
    atom1 = atom1.strip()
    atom2 = atom2.strip()
    BOND_LIST_FULL["backbone"].append((atom1, atom2))


BOND_LIST_FULL["A"] = [("C1'", "N9"),
                       ("N9",  "C8"),
                       ("C8",  "N7"),
                       ("N7",  "C5"),
                       ("C5",  "C4"),
                       ("C4",  "N9"),
                       ("C5",  "C6"),
                       ("C6",  "N1"),
                       ("N1",  "C2"),
                       ("C2",  "N3"),
                       ("N3",  "C4"),
                       ("C6",  "N6")]


BOND_LIST_FULL["G"] = [("C1'", "N9"),
                       ("N9",  "C8"),
                       ("C8",  "N7"),
                       ("N7",  "C5"),
                       ("C5",  "C4"),
                       ("C4",  "N9"),
                       ("C5",  "C6"),
                       ("C6",  "N1"),
                       ("N1",  "C2"),
                       ("C2",  "N3"),
                       ("N3",  "C4"),
                       ("C6",  "O6"),
                       ("C2",  "N2")]

BOND_LIST_FULL["C"] = [("C1'", "N1"),
                       ("N1",  "C2"),
                       ("C2",  "N3"),
                       ("N3",  "C4"),
                       ("C4",  "C5"),
                       ("C5",  "C6"),
                       ("C6",  "N1"),
                       ("C4",  "N4"),
                       ("C2",  "O2")]

BOND_LIST_FULL["U"] = [("C1'", "N1"),
                       ("N1",  "C2"),
                       ("C2",  "N3"),
                       ("N3",  "C4"),
                       ("C4",  "C5"),
                       ("C5",  "C6"),
                       ("C6",  "N1"),
                       ("C4",  "O4"),
                       ("C2",  "O2")]




import parse_pisa_xml

if True:
    if coot_gui_api.main_menubar():
        menu = coot_menubar_menu("PISA")
        submenu_pisa = gtk.Menu()
        menuitem_pisa = gtk.MenuItem("PISA Assemblies...")

        add_simple_coot_menu_menuitem(
            submenu_pisa, "PISA assemblies...",
            lambda func:
            coot_gui.molecule_chooser_gui("Choose molecule for PISA assembly analysis",
                                 lambda imol: parse_pisa_xml.pisa_assemblies(imol)))

        add_simple_coot_menu_menuitem(
            submenu_pisa, "PISA interfaces...",
            lambda func:
            coot_gui.molecule_chooser_gui("Choose molecule for PISA interface analysis",
                                          lambda imol: parse_pisa_xml.pisa_interfaces(imol)))



#!/usr/bin/env python
"""A class to access torsion mean and standard deviation information for the backbone rotamers"""

# Copyright 2010 Kevin Keating
# 
# Licensed under the Educational Community License, Version 2.0 (the
# "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
# 
# http://www.osedu.org/licenses/ECL-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.

class RotDataLoader:
    """A class to access torsion mean and standard deviation information for the backbone rotamers"""
    
    def __init__(self, filename):
        """Initialize a RotDataLoader object by reading data from a csv file
        
        ARGUMENTS:
            filename - a csv file containing data on the rotamers
        RETURNS:
            an initialized RotDataLoader object
        """
        
        input = open(filename, 'r')
        input.readline() #skip the header
        
        self.rotData = {}
        for curline in input:
            curdata = curline.split(",")
            rot = curdata[0].strip()
            if rot == "": continue #skip blank lines
            self.rotData[rot] = map(float, curdata[3:10]+curdata[13:20])
        input.close()
            
    
    
    #functions to access each mean and standard deviation
    def prevDeltaMean(self, rot): return self.rotData[rot][0]
    def epMean       (self, rot): return self.rotData[rot][1]
    def zetaMean     (self, rot): return self.rotData[rot][2]
    def alphaMean    (self, rot): return self.rotData[rot][3]
    def betaMean     (self, rot): return self.rotData[rot][4]
    def gammaMean    (self, rot): return self.rotData[rot][5]
    def curDeltaMean (self, rot): return self.rotData[rot][6]
    
    def prevDeltaSD  (self, rot): return self.rotData[rot][7]
    def epSD         (self, rot): return self.rotData[rot][8]
    def zetaSD       (self, rot): return self.rotData[rot][9]
    def alphaSD      (self, rot): return self.rotData[rot][10]
    def betaSD       (self, rot): return self.rotData[rot][11]
    def gammaSD      (self, rot): return self.rotData[rot][12]
    def curDeltaSD   (self, rot): return self.rotData[rot][13]



# from Jay, stackoverflow.com 2009
#
# CC BY-SA 2.5 license
#
# https://stackoverflow.com/questions/377017/test-if-executable-exists-in-python
#
def which(program):
    import os
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file

    return None




# the smaller the sigmas the more weight the restraints have
# This pushes up the weight a bit (c.f. 1.0)

global cho_bond_sigma_scale
cho_bond_sigma_scale = 3.0

def get_glyco_model_dir():
    import os
    return os.path.join(pkgdatadir(), "data", "cho-models")

def get_glyco_tree_acedrg_pyranose_dictionaries_dir():
    import os
    return os.path.join(pkgdatadir(), "data", "cho-acedrg")

def remove_duplicates(ls):
    return list(set(ls))

def read_acedrg_pyranose_dictionaries():
    import os
    cho_list = ["NAG", "MAN", "BMA", "SIA", "GLC", "FUC", "XYP"]
    for comp_id in cho_list:
        fn = os.path.join(get_glyco_tree_acedrg_pyranose_dictionaries_dir(),
                          comp_id + "-acedrg.cif")
        if os.path.isfile(fn):
            coot.read_cif_dictionary(fn)

def add_cho_restraints_for_residue(imol, residue_spec):
    if isinstance(residue_spec, list):
        id = glyco_tree_residue_id(imol, residue_spec)
        # print "BL DEBUG:: --------------------- add_cho_restraints_for_residue: glyco_tree_residue_id:", id
        # print "BL DEBUG:: --------------------- add_cho_restraints_for_residue: glyco_tree_residues:", coot.glyco_tree_residues_py(imol, residue_spec)
        add_cho_restraints_for_residue_with_id(imol, residue_spec, id)

def add_cho_restraints_for_residue_with_id(imol, residue_spec, glyco_id):

    # hacky?! 
    def pad_name(str):
        l = len(str)
        ret = "    "
        if (l > 3):
            return str
        else:
            return " " + str.ljust(3)

    # return list [atom_1, atom_2, d, esd]
    # or False
    def line2extra_bond_restraint_spec(parent_residue_spec, line):
        global cho_bond_sigma_scale
        parts = line.split()
        if (len(parts) == 7):
            at_name_1 = pad_name(parts[0])
            at_name_2 = pad_name(parts[1])
            mean_str = parts[2]
            sd_str = parts[3]
            n_str = parts[4]
            d_str = parts[5]
            mod_sarle_str = parts[6]
            # could be done in one?!
            mean = float(mean_str)
            sd = float(sd_str)
            n = float(n_str)
            d = float(d_str)
            mod_sarle = float(mod_sarle_str)

            if not (n >= 20):
                return False
            else:
                if not (d < 4.3):
                    return False
                else:
                    return [[residue_spec_to_chain_id(residue_spec),
                             res_spec_utils.residue_spec_to_res_no(residue_spec),
                             coot_utils.residue_spec_to_ins_code(residue_spec),
                             at_name_1, ""],
                            [residue_spec_to_chain_id(parent_residue_spec),
                             res_spec_utils.residue_spec_to_res_no(parent_residue_spec),
                             coot_utils.residue_spec_to_ins_code(parent_residue_spec),
                             at_name_2, ""],
                            mean, cho_bond_sigma_scale]

    def glyco_id2level_number(glyco_id):
        return glyco_id[0]

    def glyco_id2prime_arm_sym(glyco_id):
        return glyco_id[1]

    def glyco_id2residue_type(glyco_id):
        return glyco_id[2]

    def glyco_id2link_type(glyco_id):
        return glyco_id[3]

    def glyco_id2parent_residue_type(glyco_id):
        return glyco_id[4]

    def glyco_id2residue_spec(glyco_id):
        return glyco_id[5]


    # main line
    #
    if isinstance(glyco_id, list):
        dir = get_glyco_model_dir()
        level = glyco_id2level_number(glyco_id)
        prime_arm_sym = glyco_id2prime_arm_sym(glyco_id)
        res_name = glyco_id2residue_type(glyco_id)
        link_type = glyco_id2link_type(glyco_id)
        parent_res_name = glyco_id2parent_residue_type(glyco_id)
        parent_res_spec = glyco_id2residue_spec(glyco_id)
        fn_file = "model-level-" + str(level) + "-" + res_name + "-" + \
                  link_type + "-" + parent_res_name + ".tab"
        model_fn = os.path.join(dir, fn_file)
        if not os.path.isfile(model_fn):
            print("model file does not exist", model_fn)
        else:
            try:
                f = open(filename, 'r')
                lines = f.readlines()
                f.close()
            except:
                print("BL WARNING: couldnt open file ", filename)
                return False # with something?
            print("INFO:: read %s lines from file %s" %(lines, model_fn))
            new_restraints = [line2extra_bond_restraint_spec(line) for line in lines]
            # print "BL DEBUG:: ", new_restraints
            coot.add_extra_bond_restraints_py(imol, new_restraints)


def test_get_cho_restraints(imol):
    raw_carbo_tree_list = []

    for chain_id in coot_utils.chain_ids(imol):
        for res_serial in range(coot.chain_n_residues(chain_id, imol)):
            res_no = coot.seqnum_from_serial_number(imol, chain_id, res_serial)
            rn = coot.residue_name(imol, chain_id, res_no, "")
            if isinstance(rn, str):
                if (rn == "NAG"):
                    residue_spec = [chain_id, res_no, ""]
                    rl = coot.glyco_tree_residues_py(imol, residue_spec)
                    print("rl:", rl)
                    coot.print_glyco_tree(imol, chain_id, res_no, "")
                    if not isinstance(rl, list):
                        print("bad glyco-tree for residue", residue_spec)
                    else:
                        if (len(rl) > 3):
                            id = glyco_tree_residue_id(imol, residue_spec)
                            print("residue %s residue id %s" %(residue_spec, id))
                            add_cho_restraints_for_residue_with_id(imol,
                                                                   residue_spec,
                                                                   id)
                        else:
                            print("BL WARNING:: rl <=3", rl)

def correlation_coefficient_of_this_tree():
    with coot_utils.UsingActiveAtom(True) as [aa_imol, aa_chain_id, aa_res_no,
                                   aa_ins_code, aa_atom_name, aa_alt_conf,
                                   aa_res_spec]:
        residues = glyco_tree_residues(aa_imol, aa_res_spec)
        cc = coot.map_to_model_correlation(aa_imol, residues, [], 0,
                                      coot.imol_refinement_map())
        txt = "residues %s\ncc: %5.3f" %(residues, cc)
        print(txt)
        coot.info_dialog(txt)



# filter.py
# Copyright 2006, 2007 by Bernhard Lohkamp
# Copyright 2006, 2007 by Paul Emsley, The University of York
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Basic python function, filter the objects in list ls by function
# fn.  e.g. filter(lambda x: not x%2, [0,1,2,3]) -> [0,2]
# 
# BL says:: python has this function build in (now), so use python's
# version
#
def filter_coot(fn,ls):

  ret_ls = []
  if len(ls) > 0:
   for i in range(len(ls)):
     print(i, ls[i])
     if fn(ls[i]):
        print(i, ls[i])
        ret_ls.append(ls[i])
     else: pass
  else:
    ret_ls = ls
  return ret_ls





#    bones_reader.py
#    Copyright (C) 2008  Bernhard Lohkamp, The University of York
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

mapman_exe = os.path.join(os.getenv('HOME'), "bin/lx_mapman")

def generic_object_from_bones(bones_file):

    import operator
    from types import IntType
    
    global atom_xyz_list, bone_list, conn_list
    atom_xyz_list = []
    bone_list = []
    conn_list = []
    #save_conn = False

    def add_ordinate(s):
        global atom_xyz_list
        obj = list(map(float, s.split()))
        atom_xyz_list.append(obj)

    def add_bone(s):
        global bone_list
        obj = list(map(int, s.split()))
        bone_list += obj

    #def add_connection_pairs(s):
    #    conn_list += s.split()

    def add_connection(s):
        global conn_list
        obj = list(map(int, s.split()))
        conn_list += obj     

    def my_indexer(n):
        # un-fortran indexing
        if (n % 2 == 0):
            # even
            ret = n // 2
        else:
            ret = (n - 1) // 2
        return ret - 1

    # main body
    if (os.path.isfile(bones_file)):
        fin = open(bones_file, 'r')
        lines = fin.readlines()
        fin.close()
        reading_atoms_flag = False
        reading_atom_bone_flag = False
        reading_connectivity_flag = False
        reading_atom_colour_flag = False
        reading_residue_name_flag = False
        reading_residue_type_flag = False
        reading_residue_pointers_flag = False
        
        for line in lines:
            
            # atom colour
            if ("SKL_ATOM_COLOUR" in line):
                print("found atom_colour.")
                reading_connectivity_flag = False
                #saved_conn = False

            # connectivity
            if (reading_connectivity_flag):
                add_connection(line)
            if ("SKL_CONNECTIVITY" in line):
                print("found atom_connectivity.")
                reading_connectivity_flag = True
                reading_atom_bone_flag = False

            # bones
            if (reading_atom_bone_flag):
                add_bone(line)
            if ("SKL_ATOM_BONE" in line):
                print("found atom_bone.")
                reading_atoms_flag = False
                reading_atom_bone_flag = True

            # atoms
            if (reading_atoms_flag):
                add_ordinate(line)
            if ("SKL_ATOM_XYZ" in line):
                print("found atom_xyz")
                reading_atoms_flag = True

        print("length atom_xyz_list: %s length bone_list: %s length conn_list: %s" \
              %(len(atom_xyz_list), len(bone_list), len(conn_list)))

        lines_obj = coot.new_generic_object_number("Lines")

        current_position = False
        for connections in conn_list:
            if (not current_position):
                #print "(from %s) init move to : %s" %(connections,
                #                                      my_indexer(connections))
                current_position = my_indexer(connections)
            else:
                xyz_index = my_indexer(connections)
                if (connections % 2 == 0):
                    # even connections
                    # a move to:
                    #print "(from %s) move to: %s" %(connections, xyz_index)
                    current_position = xyz_index
                else:
                    # a draw to:
                    #print "(from %s) line: %s %s" %(connections,
                    #                                current_position,
                    #                                xyz_index)
                    coot.to_generic_object_add_line(lines_obj, "green", 3,
                                               atom_xyz_list[current_position][0],
                                               atom_xyz_list[current_position][1],
                                               atom_xyz_list[current_position][2],
                                               atom_xyz_list[xyz_index][0],
                                               atom_xyz_list[xyz_index][1],
                                               atom_xyz_list[xyz_index][2])
                    current_position = my_indexer(connections)

        coot.set_display_generic_object(lines_obj, 1)
            
    else:
        print("BL INFO:: no bones file %s found" %bones_file)


def bones_it(map_file_name):
    bones_file = "my.bones"
    data_lines = ["read m1 " + map_file_name + " ccp4",
                  "bo sk m1 0.5 0.15 1",
                  "bones connect",
                  bones_file,
                  "skl",
                  "5",
                  "quit"]

    coot_utils.popen_command(mapman_exe, [], data_lines, "coot-mapman.log", True)
    # now read in bones_file
    generic_object_from_bones(bones_file)

# add a menu item:
menu = coot_gui.coot_menubar_menu("Mapman")

def bonesing_func(imol):
    print("bonesing", imol)
    coot.export_map(imol, "tmp.map")
    bones_it("tmp.map")
    # remove tmp map?
    if (os.path.isfile("tmp.map")):
        print("BL INFO:: removing temporary map file tmp.map")
        os.remove("tmp.map")

add_simple_coot_menu_menuitem(menu, "Mapman Bones...",
                              lambda func: coot_gui.map_molecule_chooser_gui("Map to Bonesify:",
                                                       lambda imol: bonesing_func(imol)))

#generic_object_from_bones("my.bones")




import coot
import coot_gui
import coot_utils
import os

global pisa_command
pisa_command = "pisa"
global pisa_min_version
pisa_min_version = "v1.13"

def pisa_molecule_chooser_gui(mode):

    if mode == "interfaces":
        coot_gui.molecule_chooser_gui("Choose molecule for PISA assembly analysis",
                                      lambda imol: pisa_interfaces(imol))

    if mode == "assemblies":
        coot_gui.molecule_chooser_gui("Choose molecule for PISA assembly analysis",
                                      lambda imol: pisa_assemblies(imol))


def pisa_assemblies(imol):

    global pisa_min_version
    pisa_exe = pisa_new_enough_qm()
    if not pisa_exe:
        msg = "Your pisa version it too old.  Need at least " + pisa_min_version + "."
        coot.info_dialog(msg)
    else:
        #
        # main line
        pdb_file_name, pisa_config, pisa_xml_file_name = prep_for_pisa('assemblies', imol)

        status_1 = coot_utils.popen_command(pisa_exe,
                                 ["pisa", "-analyse", pdb_file_name],
                                 [], "pisa.log", False)

        if (status_1 != 0):
            coot.info_dialog("Ooops PISA failed to deliver the goods!\n\n(Go for a curry instead?)")
        else:
            # good
            # used to be print "BL DEBUG:: 2nd pisa args", [pisa_project_name, "-xml", pisa_config]
            status_2 = coot_utils.popen_command(pisa_exe,
                                     ["pisa", "-xml", "assemblies"],
                                     [], pisa_xml_file_name, False)
            if (status_2 == 0):
                pisa_assemblies_xml(imol, pisa_xml_file_name)


# called by pisa_assemblies, which is the entry point from the main
# program gui.
#
#  it calls parse_pisa_assemblies which does the work

def pisa_assemblies_xml(imol, file_name):

    from xml.etree.ElementTree import ElementTree
    import os

    if (os.path.isfile(file_name)):
        print("opened", file_name)
        sm = ElementTree()
        xml_file = clean_xml_file(file_name)
        if xml_file:
            sm.parse(xml_file)
            if (xml_file != file_name):
                os.remove(xml_file)
            parse_pisa_assemblies(imol, sm)
            # rm xml_file ?!

# return a filename which is clean (pisa) xml or False if there was problems
# not sure if needed any more if v1.13
def clean_xml_file(filename):

    if (os.path.isfile(filename)):
        fin = open(filename, 'r')
        lines = fin.readlines()
        fin.close()
        if ("<pisa_" == lines[0][0:6] or
            "<pdb_" == lines[0][0:5]):
            # have a clean file already, return filename
            return filename
        else:
            # not a clean xml file, so clean it up
            new_lines = []
            start_write = 0
            for line in lines:
                if ("<pisa_" == line[0:6] or
                    "<pdb_" == line[0:5]):
                    start_write += 1
                if (start_write == 1):
                    new_lines.append(line)
                if (start_write == 2):
                    new_lines.append(line)
                    break
            if new_lines:
                tmp_file = "xml-tmp.xml"
                fout = open(tmp_file, 'w')
                fout.writelines(new_lines)
                fout.close()
                return tmp_file
            else:
                return False

    else:
        return False

    
# Exported to the main level.  A molecule is common to assemblies and interfaces.
# 
# If pisa-result-type is 'assemblies', return a molecule number or False
# 
# If pisa-result-type is 'interfaces', return a interface-molecule record or False
#
# If pisa-result-type is neither of the above, return False
#
# a interface-molecule record contains information about pvalue and residues.
#
def pisa_handle_xml_molecule(imol, molecule, pisa_results_type):

    # was it a chain? (or residue selection disguised as
    # a chain?)
    #
    def pisa_make_atom_selection_string(chain_id_raw):

        #  first try to split the chain-id-raw on a "]".  If there was no
        # "]" then we had a simple chain-id.  If there was a "]" then we
        # have something like "[CL]D:32", or "[ZN]-:2" from which we need
        # to extract a residue number and a chain id.  Split on ":" and
        # construct the left and right hand sides of s.  Then use those
        # together to make an mmdb selection string. If chaind-id is "-"
        # then reset it to blank. 
        #
        if ("]" in chain_id_raw):
            # print "found a residue selection chain_id", chain_id_raw
            s = chain_id_raw[chain_id_raw.find("]") + 1:] # e.g. "D:32"
            if (":" in s):
                residue_string = s[s.find(":") + 1:]
                chain_string   = s[0:s.find(":")]
                element_string = chain_id_raw[1:chain_id_raw.find("]")]
                if (chain_string == "-"):
                    atom_selection_string = "// /" + residue_string + "/" + element_string
                else:
                    atom_selection_string = "//" + chain_string + "/" + residue_string
                # print "BL coot_utils.debug:: atom_selection_string: %s from %s" \
                #      %(atom_selection_string, chain_id_raw)
                return atom_selection_string
        else:
            #print "found a simple chain_id", chain_id_raw
            return "//" + chain_id_raw

    # return a list of residue dictionaries
    # filter out the ones with bsa < 0.1
    # residues_xml are parsed as a list of xml elements
    #
    def handle_residues(residues_xml):
        residues = []
        for residue in residues_xml:
            res_dic = dict((ele.tag, ele.text) for ele in residue)
            if not res_dic["ins_code"]:
                res_dic["ins_code"] = ""
            if (float(res_dic["bsa"]) > 0.1):
                residues.append(res_dic)
        return residues


    ( STRING,
      STDOUT) = list(range(2))
    # record in an element in xml element tree (molecule)
    # port is STRING or STDOUT
    #
    def print_molecule(record_xml, port):
        # first make a dictionary
        rec_dic = dict((ele.tag, ele.text) for ele in record_xml)
        # rec_dic = record
        # now either print to stdout or write to string (which will
        # be returned - otherwise return None, False on error)
        ret = None
        if (port == STRING):
            import io
            out = io.StringIO()
        elif (port == STDOUT):
            import sys
            out = sys.stdout
        else:
            return False
        
        out.write("[molecule: id %s\n"       %rec_dic['id'])
        out.write(" molecule: chain-id %s\n" %rec_dic['chain-id'])
        out.write(" molecule: class %s\n"    %rec_dic['class'])
        out.write(" molecule: symop %s\n"    %rec_dic['symop'])
        out.write(" molecule: symop-no %s\n" %rec_dic['symop-no'])
        out.write(" molecule: natoms %s\n"   %rec_dic['natoms'])
        out.write(" molecule: nres %s\n"     %rec_dic['nres'])
        out.write(" molecule: area %s\n"     %rec_dic['area'])
        out.write(" molecule: solv-en %s\n"  %rec_dic['solv-en'])
        out.write(" molecule: pvalue %s\n"   %rec_dic['pvalue'])
        out.write(" molecule: with %s residues]\n" %len(record_xml.getiterator("residues")))
        if (port == STRING):
            ret = out.getvalue()
            out.close()
        return ret

    # record in an element in xml element tree (residue)
    def print_residue(record_xml, port):
        # first make a dictionary
        rec_dic = dict((ele.tag, ele.text) for ele in record_xml)
        # rec_dic = record
        # now either print to stdout or write to string (which will
        # be returned - otherwise return None, False on error)
        ret = None
        if (port == STRING):
            import io
            out = io.StringIO()
        elif (port == STDOUT):
            import sys
            out = sys.stdout
        else:
            return False
        
        out.write("[residue: ser-no %s,\n" %rec_dic['ser-no'])
        out.write(" name %s,\n"            %rec_dic['nam'])
        out.write(" seq-num %s,\n"         %rec_dic['seq-num'])
        out.write(" ins-code %s,\n"        %rec_dic['ins-code'])
        out.write(" asa %s,\n"             %rec_dic['asa'])
        out.write(" bsa %s,\n"             %rec_dic['bsa'])
        out.write(" solv-en %s]\n"         %rec_dic['solv-en'])
        if (port == STRING):
            ret = out.getvalue()
            out.close()
        return ret

    #
    def process_molecule_internal(molecule_xml):
        
        # molecule is an element in the xml tree
        # lets make a dictionary out of it for easier handling
        mol_dic = dict((ele.tag, ele.text) for ele in molecule)
        #
        # rtop-symbols are common to interfaces and assemblies
        rtop_symbols = ['rxx', 'rxy', 'rxz', 'ryx', 'ryy', 'ryz', 'rzx', 'rzy', 'rzz',
                        'tx', 'ty', 'tz']
        # matrix elements
        # these symbols only interfaces have
        extra_symbols = ['pvalue', 'residues']

        ass_rtop_symbols = []        # association
        atom_selection_string = "//" # default, everything
        symm_name_part = ""          # the atom selection info without
                                     # "/" because that would chop the
                                     # name in the display manager.


        chain_id_raw = mol_dic["chain_id"]
        atom_selection_string = pisa_make_atom_selection_string(chain_id_raw)
        if (chain_id_raw == atom_selection_string):
            symm_name_part = "chain " + chain_id_raw
        else:
            symm_name_part = chain_id_raw

        # was it on of the rotation or coot_utils.translation symbols?
        for symbol in rtop_symbols:
            ass_rtop_symbols.append([symbol, float(mol_dic[symbol])])

        # do something with residues (if interface)
        if "residues" in mol_dic:
            residues = molecule.getiterator("residue")
            mol_dic["residues"] = handle_residues(residues)
            
        if not (len(ass_rtop_symbols) == 12):
            return False # ass_rtop_symbols were not all set
        else:
            mat = [sym[1] for sym in ass_rtop_symbols]
            #print "== atom-selection string %s   mat:::: %s" %(atom_selection_string, mat)
            #print "currently %s molecules" %(coot.graphics_n_molecules())
            new_molecule_name = "Symmetry copy of " + \
                                str(imol) +\
                                " using " + symm_name_part
            # new-molecule-by-symop-with-atom-selection,
            # perhaps? (20100222 doesn't seem needed because
            # the transformation is contained in mat.)
            new_mol_no = coot.new_molecule_by_symmetry_with_atom_selection(
                imol,
                new_molecule_name,
                atom_selection_string,
                *(mat + [0,0,0]))

            # the return value depends on pisa-results-type
            if pisa_results_type == 'assemblies':
                # assemblies:
                return new_mol_no
            elif pisa_results_type == 'interfaces':
                # interfaces:
                # in python list of 3 (2 would be enough?!)
                return [new_mol_no, mol_dic["symop"],
                        mol_dic]
            else:
                return False
                
    # main line
    #
    if (pisa_results_type == 'assemblies' or
        pisa_results_type == 'interfaces'):
        return process_molecule_internal(molecule)
    else:
        return False


# ----------------------------------------------------
#                      pisa assemblies:
# ----------------------------------------------------
#
# pisa results
#    name
#    status
#    total_asm
#    asm_set
#       ser_no
#       assembly
#          id
#          size
#          mmsize
#          diss_energy
#          asa
#          bas
#          entropy
#          diss_area
#          int_energy
#          n_uc
#          n_diss
#          symNumber
#          molecule
#             chain_id
#             rxx
#             rxy
#             rxz
#             tx
#             ryx
#             ryy
#             ryz
#             ty
#             rzx
#             rzy
#             rzz
#             tz
#             rxx-f
#             rxy-f
#             rxz-f
#             tx-f
#             ryx-f
#             ryy-f
#             ryz-f
#             ty-f
#             rzx-f
#             rzy-f
#             rzz-f
#             tz-f

def parse_pisa_assemblies(imol, entity):

    # Return the model number of the new assembly molecule
    #
    def create_assembly_set_molecule(assembly_set_molecule_numbers,
                                     assembly_set_number = ""):
        if not assembly_set_molecule_numbers:
            return False
        else:
            first_copy = coot.copy_molecule(assembly_set_molecule_numbers[0])
            if not coot_utils.valid_model_molecule_qm(first_copy):
                return False
            else:
                rest = assembly_set_molecule_numbers[1:]
                merge_molecules(rest, first_copy)
                coot.set_molecule_name(first_copy, "Assembly Set " + assembly_set_number)
                return first_copy

    ( STRING,
      STDOUT) = list(range(2))
    # record in an element in xml element tree
    # port is STRING or STDOUT
    #
    def print_assembly(record, port):
        # first make a dictionary
        #rec_dic = dict((ele.tag, ele.text) for ele in record)
        rec_dic = record
        # now either print to stdout or write to string (which will
        # be returned - otherwise return None, False on error)
        ret = None
        if (port == STRING):
            import io
            out = io.StringIO()
        elif (port == STDOUT):
            import sys
            out = sys.stdout
        else:
            return False
        
        out.write("assembly id: %s\n" %rec_dic['id'])
        out.write("assembly size: %s\n" %rec_dic['size'])
        out.write("assembly symm-number: %s\n" %rec_dic['symNumber'])
        out.write("assembly asa: %s\n" %rec_dic['asa'])
        out.write("assembly bsa: %s\n" %rec_dic['bsa'])
        out.write("assembly diss_energy: %s\n" %rec_dic['diss_energy'])
        out.write("assembly entropy: %s\n" %rec_dic['entropy'])
        out.write("assembly diss_area: %s\n" %rec_dic['diss_area'])
        out.write("assembly int_energy: %s\n" %rec_dic['int_energy'])
        # maybe split next line when more than 2 or so molecules?!
        out.write("assembly components: %s\n" %rec_dic['molecule'])
        if (port == STRING):
            ret = out.getvalue()
            out.close()
        return ret

    # Return a list of model numbers
    #
    def create_assembly_molecule(assembly_molecule_numbers):
        return assembly_molecule_numbers

    # return an assembly record:
    # this is now a dictionary with all assembly properties
    # molecule is replaced with a list of symmetry created molecules
    # arg assembly is assembly element from xml tree
    #
    def handle_assembly(assembly):
        assembly_molecule_numbers = []
        assembly_dic = dict((ele.tag, ele.text) for ele in assembly)
        molecules = assembly.getiterator("molecule")
        for molecule in molecules:
            mol_no = pisa_handle_xml_molecule(imol, molecule, 'assemblies')
            assembly_molecule_numbers.append(mol_no)
        assembly_dic["molecule"] = assembly_molecule_numbers
        for mol_no in assembly_molecule_numbers:
            if (coot_utils.valid_model_molecule_qm(mol_no)):
                coot.set_mol_displayed(mol_no, 0)
        return assembly_dic


    # handle assembly-set (the xml tree element for asm_set.)
    #
    # return values [False or the molecule number of the
    # assembly-set] and the assembly-record-set (in dictionary
    # format - which includes the assembly set molecules list!!
    #
    def handle_assembly_set(assembly_set):
        assembly_records = assembly_set.find("assembly")
        assembly_record = handle_assembly(assembly_records)
        assembly_set_molecules = assembly_record["molecule"]
        new_mol = create_assembly_set_molecule(assembly_set_molecules,
                                               assembly_record["id"])

        return new_mol, assembly_record

    # main line
    #

    #
    first_assembly_set_is_displayed_already_qm = False
    top_assembly_set = False

    assemblies = entity.getiterator("asm_set")
    for ass in assemblies:
        molecule_number, assembly_record_set = handle_assembly_set(ass)
        if not top_assembly_set:
            # make a string out of the dictionary
            top_assembly_set = print_assembly(assembly_record_set, STRING)
        if first_assembly_set_is_displayed_already_qm:
            coot.set_mol_displayed(molecule_number, 0)
        else:
            first_assembly_set_is_displayed_already_qm = True


    if top_assembly_set:
        print("top assembly-set:\n", top_assembly_set)
        s = "top assembly-set: \n\n" + top_assembly_set
    else:
        s = "no assembly-sets found"
    coot.info_dialog(s)
            

def make_pisa_config(pisa_coot_dir, config_file_name):
    s = os.getenv("CCP4")
    if (os.path.isdir(s)):
        ls = [["DATA_ROOT", os.path.join(s, "share", "pisa")],
              ["SRS_DIR", os.path.join(s, "share", "ccp4srs")],
              ["PISTORE_DIR", os.path.join(s, "share", "pisa")],
              ["PISA_WORK_ROOT", pisa_coot_dir],
              # according to Paule (and I agree):
              # that we need these next 3 is ridiculous
              # BL:: hope not really needed (as for Win exe is missing)
              # maybe already fixed!?
              ["MOLREF_DIR", os.path.join(s, "share", "pisa")],
              ["RASMOL_COM", os.path.join(s, "bin", "rasmol")],
              ["CCP4MG_COM", os.path.join(s, "bin", "ccp4mg")],
              # the parent dir of this needs to be writable, usually isnt
              # since it's all in a ccp4 install dir (which is DATA_ROOT?!?!)
              ["SESSION_PREFIX", "pisa_"],
              ]
        fin = open(config_file_name, 'w')
        for item_pair in ls:
            fin.write(item_pair[0] + "\n")
            fin.write(item_pair[1] + "\n")
        fin.close()


# 20100213 prep-for-pisa needs to make directory, config file, write
# the pdb file and the return value should be #f if there was a
# problem or some value where we can check that pisa -analyse ran
# (probably a directory).  It is not clear right now where the output
# is going.  config files has PISA_WORK_ROOT coot-pisa but things
# seems to be written to DATA_ROOT
# /home/emsley/ccp4/ccp4-6.1.3/share/pisa which seems like a bug (or
# something like it) in pisa.  Needs rechecking
#
# maybe santisation of xml fiel can go here too?!?!
def prep_for_pisa(mode, imol):

    #
    def make_stubbed_name(imol):
        return coot_utils.strip_extension(os.path.basename(coot.molecule_name(imol)))

    if coot_utils.valid_model_molecule_qm(imol):
        pisa_coot_dir = "coot-pisa"
        stubbed_name = make_stubbed_name(imol)
        pdb_file_name      = os.path.join(pisa_coot_dir, stubbed_name + ".pdb")
        pisa_config        = os.path.join(pisa_coot_dir, stubbed_name + "-pisa.cfg")
        pisa_xml_file_name = os.path.join(pisa_coot_dir, stubbed_name + "-" + str(mode) + ".xml")
        #pisa_project_name  = os.path.join(stubbed_name)

        coot.make_directory_maybe(pisa_coot_dir)
        make_pisa_config(pisa_coot_dir, pisa_config)
        coot.write_pdb_file(imol, pdb_file_name)
        if (os.path.isfile(pdb_file_name)):
            return pdb_file_name, pisa_config, pisa_xml_file_name
        else:
            return False, False, False

# needs fleshing out (see notes for prep-for-pisa).
#
def cached_pisa_analysis(dir):
    return False
    
#
# return pisa_command_exe or False
#
def pisa_new_enough_qm():
    global pisa_command
    global pisa_min_version
    if not pisa_command:
        pisa_command="pisa"
    pisa_exe = coot_utils.find_exe(pisa_command, "CBIN", "CCP4_BIN", "PATH")
    if pisa_exe:
        tmp_file = "pisa-version.log"
        process = coot_utils.popen_command(pisa_exe, [], [], tmp_file)
        fin = open(tmp_file, 'r')
        lines = fin.readlines()
        fin.close()
        os.remove(tmp_file)
        for line in lines:
            if " v" in line:
                if line.split()[0] >= pisa_min_version:
                    return pisa_exe
                else:
                    return False
    else:
        return False
    

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
#                                   interfaces
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------

def pisa_interfaces(imol):
    global pisa_min_version
    pisa_exe = pisa_new_enough_qm()
    if not pisa_exe:
        msg = "Your pisa version it too old.  Need at least" \
              + pisa_min_version + "."
        coot.info_dialog(msg)
    else:
        pdb_file_name, pisa_config, pisa_xml_file_name = prep_for_pisa("interfaces", imol)

        if pisa_config:
            if not cached_pisa_analysis(pisa_config):
                # pisa analysis
                status_1 = coot_utils.popen_command(pisa_exe,
                                         ["pisa", "-analyse", pdb_file_name],
                                         [],
                                         "pisa-analysis.log", False)
                if (status_1 == 0):
                    status_2 = coot_utils.popen_command(pisa_exe,
                                             ["pisa", "-xml", "interfaces"],
                                             [],
                                             pisa_xml_file_name, False)
                    if (status_2 == 0):  # good
                        pisa_interfaces_xml(imol, pisa_xml_file_name)


def pisa_interfaces_xml(imol, file_name):
    if not os.path.isfile(file_name):
        print("WARNING:: in pisa_interfaces_xml: %s does not exist" %file_name)
    else:
        from xml.etree.ElementTree import ElementTree
        sm = ElementTree()
        xml_file = clean_xml_file(file_name)
        if xml_file:
            sm.parse(xml_file)
            if (xml_file != file_name):
                os.remove(xml_file)
            parse_pisa_interfaces(imol, sm)

# pdb_entry
#    pdb_code
#    status
#    n_interfaces
#    interface
#       id
#       type
#       n_occ
#       int_area
#       int_solv_en
#       pvalue
#       stab_en
#       css
#       overlap
#       x-rel
#       fixed
#       h-bonds
#          n_bonds
#          bond
#             chain-1
#             res-1
#             seqnum-1
#             inscode-1
#             atname-1
#             chain-2
#             res-2
#             seqnum-2
#             inscode-2
#             atname-2
#             dist
#       salt-bridges
#          n-bonds
#          bond
#             chain-1
#             res-1
#             seqnum-1
#             inscode-1
#             atname-1
#             chain-2
#             res-2
#             seqnum-2
#             inscode-2
#             atname-2
#             dist
#       ss-bonds
#          n-bonds
#          bond
#             chain-1
#             res-1
#             seqnum-1
#             inscode-1
#             atname-1
#             chain-2
#             res-2
#             seqnum-2
#             inscode-2
#             atname-2
#             dist
#       cov-bonds
#          n-bonds
#          bond
#             chain-1
#             res-1
#             seqnum-1
#             inscode-1
#             atname-1
#             chain-2
#             res-2
#             seqnum-2
#             inscode-2
#             atname-2
#             dist
#       molecule
#          id
#          chain_id
#          class
#          symop
#          symop_no
#          cell_i
#          cell_j
#          cell_k
#          rxx
#          rxy
#          rxz
#          tx
#          ryx
#          ryy
#          ryz
#          ty
#          rzx
#          rzy
#          rzz
#          tz
#          int_natoms
#          int_nres
#          int_area
#          int_solv_en
#          pvalue
#          residues
#             residue
#                ser_no
#                name
#                seq_num
#                ins_code
#                bonds
#                asa
#                bsa
#                solv_en
#                
def parse_pisa_interfaces(imol, xml_entity):
    
    # return a list of bonds:
    #
    # [bond-type, atom-spec-1, atom-spec-2]
    # 
    # where bond-type is 'hbond', 'salt-bridge', 'ss-bond', or 'cov-bond'.
    #
    def molecule_bonds(entity):

        # return a list of 2 atom specs given something like: 
        #
        # xml entity bond
        # return False if the atom specs are not fully set
        #
        def parse_bond(bond_description):

            bond_dic = dict((ele.tag, ele.text) for ele in bond_description)
            chain_id_1  = bond_dic["chain-1"]
            chain_id_2  = bond_dic["chain-2"]
            res_no_1    = bond_dic["seqnum-1"]
            res_no_2    = bond_dic["seqnum-2"]
            ins_code_1  = bond_dic["inscode-1"]
            ins_code_2  = bond_dic["inscode-2"]
            atom_name_1 = bond_dic["atname-1"]
            atom_name_2 = bond_dic["atname-2"]

            if not (chain_id_1 and chain_id_2 and
                    res_no_1 and res_no_2 and
                    atom_name_1 and atom_name_2):
                return False
            else:
                if not ins_code_1:
                    ins_code_1 = ""
                if not ins_code_2:
                    ins_code_2 = ""
                return [[chain_id_1, int(res_no_1), ins_code_1, atom_name_1, ""],
                        [chain_id_2, int(res_no_2), ins_code_2, atom_name_2, ""]]
            
                    
        ret_bonds = []
        bond_type = entity.tag
        bonds = entity.getiterator("bond")
        for bond in bonds:
            coot_utils.atom_specs = parse_bond(bond)
            if coot_utils.atom_specs:
                coot_utils.atom_specs.insert(0, bond_type)
                ret_bonds.append(atom_specs)
        return ret_bonds
            

    def pisa_handle_xml_interface(interface_entity):
        molecules = []
        bonds = []

        # molecule info is either False or a pair of a new
        # molecule number and a pisa molecule record (which contains
        # things like pvalue, residues, id, class).
        for molecule in interface_entity.getiterator("molecule"):
            molecule_info = pisa_handle_xml_molecule(imol, molecule, 'interfaces')
            if molecule_info:
                molecules.append(molecule_info)
        for bond_type in ['h-bonds', 'salt-bridges', 'ss-bonds', 'cov-bonds']:
            bonds += (molecule_bonds(interface_entity.find(bond_type)))
        pvalue  = interface_entity.find("pvalue").text
        area    = interface_entity.find("int_area").text
        solv_en = interface_entity.find("int_solv_en").text
        stab_en = interface_entity.find("stab_en").text

        return [molecules, bonds, area, solv_en, pvalue, stab_en]

    def pisa_handle_pdb_entry(pdb_entry_entity):
        interfaces = []
        for interface_xml in pdb_entry_entity.getiterator("interface"):
            interface = pisa_handle_xml_interface(interface_xml)
            if interface:
                interfaces.append(interface)
        return interfaces
    
    # main line
    #
    pdb_entries = xml_entity.getiterator("pdb_entry")
    for pdb_entry_xml in pdb_entries:
        pdb_entry = pisa_handle_pdb_entry(pdb_entry_xml)
        handle_pisa_interfaces(pdb_entry)
        
        



# This script is an attempt to use the XML output of doxygen to
# create a python file that contains stubs of functions
# which can then be used to generate python documentation
# with Sphinx.
#
# It does some of that work, but to get it working properly
# will be about two weeks of work - or so.
# And spending that time to save maybe 3 hours of boring
# editing is not worth it.
#
# It is better to just create a python stub script now
# and modify it by tracking changes made to the nanobinds file.

import xml.etree.ElementTree as ET
mytree = ET.parse('xml/classmolecules__container__t.xml')
myroot = mytree.getroot()

def convert_type(tt: str) -> str:
    if tt == "const ": tt = "str" # needed for a coot::colour_t
    if tt == "const std::string &": tt = "str"
    if tt == "std::string": tt = "str"
    if tt == "unsigned int": tt = "int"
    if tt == "double": tt = "float"
    if tt == 'std::vector< ': tt = "list"
    if tt == 'const std::vector< ': tt = "list"
    # if tt == 'std::pair< int, unsigned int >': tt = "tuple"
    if tt == 'const std::vector< float > &': tt = "list"
    if tt == 'std::vector< float > &': tt = "list"
    if tt == "const std::vector< std::string > &": tt = "list"
    if tt == "const std::vector< std::string > &": tt = "list"
    if tt == 'const std::map< unsigned int, std::array< float, 3 > > &': tt = "dict"
    if tt == 'const std::map< unsigned int, std::array< float, 4 > > &': tt = "dict"
    if tt == 'const std::vector< std::pair< std::string, unsigned int > > &': tt = "list"
    if tt == 'const std::vector< std::pair< std::string, unsigned int > > &': tt = "list"
    if tt == 'const std::vector< std::pair< bool, mmdb::Residue * > > &, links: const std::vector< mmdb::Link > &': tt = "list"
    if tt == 'std::vector<': tt = list
    if tt == 'const coot::residue_spec_t &': tt = 'str'
    if tt == 'const coot::colour_t &': tt = 'list'   #use the proper type later
    if tt == 'const coot::atom_spec_t &': tt = 'str'
    if tt == 'coot::residue_spec_t &': tt = 'list'
    if tt == 'std::vector< coot::api::moved_atom_t > &': tt = 'list'
    if tt == 'const std::vector< coot::api::moved_residue_t > &': tt = 'list'

    return tt

def make_paren_string(function: dict) -> str:
    if function['kind'] == "function":
        r = ""
        try:
            for idx,param in enumerate(function['params']):
                if idx > 0:
                    r += ", "
                r += param['declname']
                r += ": "
                t = convert_type(param['type'])
                r += t
            r = "(self, " + r + ")"
            return r
        except KeyError as e:
            return "(self)"
    else:
        return ""


def make_return_type(function: dict) -> str:
    # print("--- make_return_type() dict is ", function)
    return_type = ""
    rt = ""
    t = str(function['type'])
    if t == 'int':   rt = "int"
    if t == 'bool':  rt = "bool"
    if t == 'void':  rt = "None"
    if t == 'float': rt = "float"
    if t == 'std::string':  rt = "str"
    if t == 'unsigned int': rt = "int"
    if t == 'std::pair< int, unsigned int >': rt = "tuple"
    if rt:
        return_type = " -> " + rt
    return rt, return_type


def make_python_script(functions: list) -> None:

    f = open("chapi-functions.py", "w")
    f.write("class molecules_container_t:\n")
    for function in functions:
        func_name = ""
        try:
            func_name = function['name']
        except AttributeError as e:
            pass
        print("\n--- make_python_script(): Handling function: ", func_name)

        parens = ""
        def_ = ""
        if function['kind'] == "function":
            parens = make_paren_string(function)
            return_type, return_type_with_arrow = make_return_type(function)
            # print("debug args: function ", function, "made args:", parens)
            def_ = "def "
            s = f"    {def_}{function['name']}{parens}{return_type_with_arrow}:\n"
            f.write(s)

            done_brief = False
            done_detailed = False
            f.write('        """ ')
            try:
                d = function["briefdescription"]
                # print(f"debug:: briefdescription:{d}:")
                f.write(d)
                # f.write('\n')
                done_brief = True
            except KeyError as e:
                # print("No briefdescription")
                pass
            except TypeError as e:
                pass
            # maybe use a for loop for this and the above ["briefdescription", "detaileddescription"]
            try:
                d = function["detaileddescription"]
                # print(f"debug:: detaileddescription:{d}:")
                # f.write('        """ ')
                f.write(d)
                # f.write('\n')
                done_detailed = True
            except KeyError as e:
                # print("No detaileddescription")
                pass
            except TypeError as e:
                pass
            if not done_brief:
                if not done_detailed:
                    # we need some doc string for the sphinx to pick up the function
                    f.write('        Sphinx-Doc-Placeholder')
            f.write('"""')
            f.write('\n')
            if not return_type:               f.write("        pass\n")
            if return_type == "int":          f.write("        return 0\n")
            if return_type == "float":        f.write("        return 0.0\n")
            if return_type == "str":          f.write("        return 'a-string'\n")
            if return_type == "bool":         f.write("        return True\n")

        f.write("\n")
    f.close()

functions = []

for x in myroot.iter('sectiondef'):

    print("#### x:", x)
    # print(dir(x))
    h = x.find('header')
    print(h)
    if h is not None:
        ht = h.text
        print("####### header text: ", ht)
    for child in x:
        if child.tag == "memberdef":
            keep_going = True
            try:
                name = "--unset--"
                a_function = {}
                kind = child.attrib['kind']
                print("   child kind:", kind)
                a_function['kind'] = kind
                if kind == "function":
                    print("\n -----  Handling function")
                    # if function.name ==  "molecules_container_t": continue
                    # if function.name == "~molecules_container_t": continue
                for ii,ch in enumerate(child):
                    print("      ch.tag ", ii, ch.tag, ch.text, ":")
                    if ch.tag == "definition":
                        if ch.text == "molecules_container_t::~molecules_container_t":
                            keep_going = False
                            break
                            # next memberdef
                        if ch.text == "molecules_container_t::molecules_container_t":
                            keep_going = False
                            break
                    if ch.tag == "param":
                        t = ch.find("type")
                        print("   param type:", t)
                        tt = t.text
                        print("    tt: '" + str(tt) + "'")
                        dn = ch.find("declname")
                        dn = dn.text
                        print("    dn", dn)
                        if tt:
                            try:
                                a_function['params'].append({"type": tt, "declname": dn})
                            except KeyError as e:
                                a_function['params'] = [{"type": tt, "declname": dn}]
                    if ch.tag == "name":
                        name = ch.text
                        a_function["name"] = name
                    if ch.tag == "type":
                        a_function["type"] = ch.text
                    if ch.tag == "briefdescription":
                        for c in ch:
                          if c.tag == "para":
                              brief_descr = c.text
                              a_function["briefdescription"] = brief_descr
                    parts = ''
                    if ch.tag == "detaileddescription":
                        for idx,c in enumerate(ch):
                            print("     detaileddescription: item", idx, "is:", c, c.text)
                            if c.tag == "para":
                                # a para can have a parameter list and no text
                                #   descr = c.text
                                #   if descr:
                                #       print("      detailed descr", descr)
                                #       a_function["detaileddescription"] = descr
                                if c.text:
                                    if parts:
                                        parts += "\n\n        "
                                        parts += c.text
                                    else:
                                        parts = c.text
                                # print(dir(c))
                                for jj, chunk in enumerate(c):
                                    print('      chunk', jj, ":", chunk, "len:", len(chunk))
                                    if chunk.tag == "parameterlist":
                                        print("        handle parameterlist here")

                                        for pk,pchunk in enumerate(chunk):
                                               print("pk:", pk, 'pchunk', pchunk)
                                               for ck,cchunk in enumerate(pchunk):
                                                   print("ck:", ck, 'cchunk', cchunk)

                                                   if cchunk.tag == "parameternamelist":
                                                      parts += "\n"
                                                      for kk,kchunk in enumerate(cchunk):
                                                           if kchunk.tag == "parametername":
                                                              print("kchunk", kk, "text:", kchunk.text)
                                                              parts += "\n       " + " :param " + kchunk.text + ": "

                                                   if cchunk.tag == "parameterdescription":
                                                       print('len(cchunk)', len(cchunk))
                                                       for kk,kchunk in enumerate(cchunk):
                                                           if kchunk.tag == "para":
                                                              print("kchunk", kk, "text:", kchunk.text)
                                                              parts += " " + kchunk.text
                                                              for kk,pchunk in enumerate(kchunk):
                                                                  if pchunk.tag == "computeroutput":
                                                                     if pchunk.text:
                                                                        print("        computeroutput:", pchunk.text)
                                                                        parts += "`"
                                                                        parts += pchunk.text
                                                                        parts += "`"
                                                                        parts += pchunk.tail

                                    if chunk.tag == "computeroutput":
                                        print("        computeroutput:", chunk.text)
                                        if chunk.text:
                                            parts += "`"
                                            parts += chunk.text
                                            parts += "`"
                                            parts += chunk.tail
                                    if chunk.tag == "simplesect":
                                        for kk,kchunk in enumerate(chunk):
                                            # print("kk:", kk, kchunk)
                                            if kchunk.tag == "para":
                                                print("kchunk", kk, "text:", kchunk.text)
                                                #parts += "Return" + " " + kchunk.text
                                                parts += "\n\n       " + " :return: " + kchunk.text
                                                for kk,pchunk in enumerate(kchunk):
                                                    if pchunk.tag == "computeroutput":
                                                       if pchunk.text:
                                                          print("        computeroutput:", pchunk.text)
                                                          parts += "`"
                                                          parts += pchunk.text
                                                          parts += "`"
                                                          parts += pchunk.tail


                    if parts:
                        a_function["detaileddescription"] = parts
                if a_function:
                    if keep_going:
                        functions.append(a_function)

            except AttributeError as e:
                print(e)

make_python_script(functions)



#!/usr/bin/env python
"""Functions for building non-bridging phosphoryl oxygens"""

# Copyright 2010 Kevin Keating
# 
# Licensed under the Educational Community License, Version 2.0 (the
# "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
# 
# http://www.osedu.org/licenses/ECL-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.

from math import radians, cos, sin

from strucCalc import minus, plus, dotProd, crossProd, scalarProd, magnitude, normalize

#constants
PHOSBONDLENGTH = 1.485
PHOSBONDANGLE  = 119.6
INITPHOSANGLE  = 108     #the angle between the O5' and the phosphoryl oxygens in the first phosphate of a segment

def buildPhosOxy(curResAtoms, prevResAtoms):
    """Calculate non-bridging phosphoryl oxygen coordinates using the coordinates of the current and previous nucleotides
    
    ARGUMENTS:
        curResAtoms - a dictionary of the current nucleotide coordinates (i.e. the nucleotide to build the phosphoryl oxygens on) in the format atomName: [x, y, z]
        prevResAtoms - a dictionary of the previous nucleotide coordinates in the format atomName: [x, y, z]
    RETURNS:
        phosOxyCoords - a dictionary of the phosphoryl oxygen coordinates in the format atomName: [x, y, z]
        or None if the residue is missing the P, O5', or O3' atoms
    """
    
    try:
        P  = curResAtoms ["P"]
        O5 = curResAtoms ["O5'"]
        O3 = prevResAtoms["O3'"]
    except KeyError:
        return None
    
    #calculate a line from O5' to O3'
    norm = minus(O5, O3)
    
    #calculate the intersection of a plane (with normal $norm and point $P) and a line (from O5' to O3')
    #using formula from http://local.wasp.uwa.edu.au/~pbourke/geometry/planeline/
    #       norm dot (P - O3)
    # i = ------------------------
    #       norm dot (O5 - O3)
    #intersecting point = O3 + i(O5 - O3)
    i = dotProd(norm, minus(P, O3)) / dotProd(norm, minus(O5, O3))
    interPoint = plus(O3, scalarProd(i, minus(O5, O3)))
    
    #move $interPoint so that the distance from $P to $interPoint is 1.485 (the length of the P-OP1 bond)
    #we also reflect the point about P
    PIline = minus(P, interPoint) #here's is where the reflection occurs, because we do $P-$interPoint instead of $interPoint-$P
    #scaledPoint = scalarProd(1/magnitude(PIline) * PHOSBONDLENGTH, PIline)
    scaledPoint = normalize(PIline, PHOSBONDLENGTH)
    #to get the new point location, we would do P + scaledPoint
    #but we need to rotate the point first before translating it back
    
    #rotate this new point by 59.8 and -59.8 degrees to determine the phosphoryl oxygen locations
    #we rotate about the axis defined by $norm
    angle = radians(PHOSBONDANGLE / 2)
    (x, y, z) = scaledPoint
    
    #unitnorm = scalarProd( 1/magnitude(norm), norm)
    unitnorm = normalize(norm)
    (u, v, w) = unitnorm
    
    a = u*x + v*y + w*z
    
    phosOxyCoords = {}
    for (atomName, theta) in (("OP1", angle), ("OP2", -angle)):
        cosTheta = cos(theta)
        sinTheta = sin(theta)
        
        #perform the rotation, and then add $P to the coordinates
        newX = a*u + (x-a*u)*cosTheta + (v*z-w*y)*sinTheta + P[0]
        newY = a*v + (y-a*v)*cosTheta + (w*x-u*z)*sinTheta + P[1]
        newZ = a*w + (z-a*w)*cosTheta + (u*y-v*x)*sinTheta + P[2]
        
        phosOxyCoords[atomName] = [newX, newY, newZ]
    
    return phosOxyCoords



def buildInitOrTerminalPhosOxy(curResAtoms, prevResAtoms = None):
    """build phosphoryl oxygens using only the O3' or O5'atom (intended for the first or last nucleotide of a chain/segment)
    
    ARGUMENTS:
        curResAtoms - a dictionary of the current nucleotide coordinates (i.e. the nucleotide to build the phosphoryl oxygens on) in the format atomName: [x, y, z]
                      this dictionary must contain at least the phosphate
    OPTIONAL ARGUMENTS:
        prevResAtoms - a dictionary of the previous nucleotide coordinates in the format atomName: [x, y, z]
                       if provided, the curResAtoms["P"] and prevResAtoms["O3'"] will be used to place the phosphoryl oxygens
                       if not provided, the curResAtoms["P"] and curResAtoms["O5'"]
    RETURNS:
        phosOxyCoords - a dictionary of the phosphoryl oxygen coordinates in the format atomName: [x, y, z]
        or None if the residue is missing the necessary atoms
    """
    
    try:
        P  = curResAtoms ["P"]
        if prevResAtoms is not None:
            O = prevResAtoms["O3'"]
            C = prevResAtoms["C3'"]
        else:
            O = curResAtoms["O5'"]
            C = curResAtoms["C5'"]
    except KeyError:
        return None
    
    #place atom along the P-O5' bond that is the appropriate distance from P
    phosOxy = minus(O, P)
    phosOxy = normalize(phosOxy, PHOSBONDLENGTH)
    
    #define plane with C5'-O5'-P
    norm = crossProd( minus(C, O), minus(P, O))
    norm = normalize(norm)
    
    #rotate dummy atom in plane about P by INITPHOSANGLE (which is the appropriate O5'-OP1 angle)
    (x, y, z) = phosOxy
    (u, v, w) = norm
    theta = -radians(INITPHOSANGLE) #use the negative angle so that we rotate away from the C5'
    
    cosTheta = cos(theta)
    sinTheta = sin(theta)
    
    #perform the rotation
    a = u*x + v*y + w*z
    phosOxy[0] = a*u + (x-a*u)*cosTheta + (v*z-w*y)*sinTheta
    phosOxy[1] = a*v + (y-a*v)*cosTheta + (w*x-u*z)*sinTheta
    phosOxy[2] = a*w + (z-a*w)*cosTheta + (u*y-v*x)*sinTheta
    
    #rotate dummy atom about O5'-P axis by 59.8 and -59.8 degrees
    norm = minus(O, P)
    norm = normalize(norm)
    
    (x, y, z) = phosOxy
    (u, v, w) = norm
    angle = radians(PHOSBONDANGLE / 2)
    
    phosOxyCoords = {}
    for (atomName, theta) in (("OP1", angle), ("OP2", -angle)):
        cosTheta = cos(theta)
        sinTheta = sin(theta)
        
        #perform the rotation, and then add $P to the coordinates
        newX = a*u + (x-a*u)*cosTheta + (v*z-w*y)*sinTheta + P[0]
        newY = a*v + (y-a*v)*cosTheta + (w*x-u*z)*sinTheta + P[1]
        newZ = a*w + (z-a*w)*cosTheta + (u*y-v*x)*sinTheta + P[2]
        
        phosOxyCoords[atomName] = [newX, newY, newZ]
    
    return phosOxyCoords
    
    



import datetime

def proc():
    fn = "../menu-items-to-fix"
    # f = open("../rel-todo-gtk3")
    f = open(fn)
    lines = f.readlines()
    add_to_menu_items = False
    add_to_other_items = False
    add_to_done_menu_items = False
    add_to_done_other_items = False
    n_menu_items  = 0
    n_other_items = 0
    n_done_menu_items  = 0
    n_done_other_items = 0
    for line in lines:
        if "--- Menu Items ---" in line:
            add_to_menu_items = True
            add_to_done_menu_items = False
            add_to_other_items = False
            add_to_done_other_items = False
        if "--- Done Menu Items ---" in line:
            add_to_menu_items = False
            add_to_done_menu_items = True
            add_to_other_items = False
            add_to_done_other_items = False
        if "--- Other Items ---" in line:
            add_to_menu_items = False
            add_to_done_menu_items = False
            add_to_other_items = True
            add_to_done_other_items = False
        if "--- Done Other Items ---" in line:
            add_to_menu_items = False
            add_to_done_menu_items = False
            add_to_other_items = False
            add_to_done_other_items = True
        arr = line.split()
        try:
            idx = arr.index(":::")
            if idx < len(arr) - 1:
                estimated_time_for_task_string = arr[idx+1]
                etft = float(estimated_time_for_task_string)
                if add_to_menu_items:
                    n_menu_items += etft
                if add_to_done_menu_items:
                    n_done_menu_items += etft
                if add_to_other_items:
                    n_other_items += etft
                if add_to_done_other_items:
                    n_done_other_items += etft
        except ValueError as e:
            pass

    return (n_menu_items, n_done_menu_items, n_other_items, n_done_other_items)


(n_menu_items, n_done_menu_items, n_other_items, n_done_other_items) = proc()


start = datetime.datetime(2021, 9, 29)
now   = datetime.datetime.now()

delta_s = int(now.strftime("%s")) - int(start.strftime("%s"))
delta_d = delta_s/(60*60*24)
# print(delta_d, n_above, n_below, n_total)
print("{:.3f} {} {:.2f} {} {}".format(delta_d, n_menu_items, n_done_menu_items, n_other_items, n_done_other_items))



# 07_ncs.py
# Copyright 2008 by The University of York
# Author: Bernhard Lohkamp
# Copyright 2007, 2008 by The University of Oxford
# Author: Paul Emsley

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or (at
# your option) any later version.
 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA

import unittest
import os
import coot
import coot_utils
import coot_ncs # coot.ncs one day
import coot_testing_utils


def insulin_res():
    return os.path.join(coot_testing_utils.unittest_data_dir, "insulin.res")
def imol_insulin():
    return coot.read_pdb(insulin_res())

class TestNCSFunctions(unittest.TestCase):


    def test01_1(self):
            """NCS maps test"""

            imol_rnase = coot.read_pdb(coot_testing_utils.rnase_pdb())
            imol_rnase_map = coot.make_and_draw_map(coot_testing_utils.rnase_mtz(), "FWT","PHWT","",0,0)
            self.assertTrue(coot_utils.valid_model_molecule_qm(imol_rnase), "imol_rnase not valid")

            self.assertTrue(coot_utils.valid_map_molecule_qm(imol_rnase_map), "imol_rnase_map not valid")

            n_mols = coot.graphics_n_molecules()
            # try to make it trip up by doing it twice:
            imol_map_2 = coot.make_and_draw_map(coot_testing_utils.rnase_mtz(), "FWT", "PHWT", "", 0 ,0)
            coot.make_dynamically_transformed_ncs_maps(imol_rnase, imol_rnase_map, 0)
            coot.make_dynamically_transformed_ncs_maps(imol_rnase, imol_map_2, 0)
            # 2*2 + 1 new maps should have been made
            n_new = coot.graphics_n_molecules()
            self.assertEqual(n_new, (n_mols + 5), "no match in number of molecules %s %s" %(n_mols, n_new))


    def test02_0(self):
        """NCS chains info"""

        # should return False
        ncs_chain_info = coot.ncs_chain_ids_py(-1)
        self.assertFalse(ncs_chain_info, "   Fail: ncs-chains returns %s, should be False" %ncs_chain_info)

        # a normal case
        imol_rnase = coot.read_pdb(coot_testing_utils.rnase_pdb())
        coot.make_ncs_ghosts_maybe(imol_rnase)
        ncs_chain_info = coot.ncs_chain_ids_py(imol_rnase)
        self.assertTrue(ncs_chain_info, "   Fail: ncs-chain-ids returns False")
        self.assertTrue(len(ncs_chain_info) > 0, "   Fail: ncs-chains returns %s" %ncs_chain_info)
        first_ghost = ncs_chain_info[0]
        self.assertTrue(len(first_ghost) > 1, "    Fail: first-ghost %s" %first_ghost)
        print("   NCS info: ", ncs_chain_info)


    def test03_0(self):
        """NCS deviation info"""

        # should return False
        ncs_chain_info = coot.ncs_chain_differences_py(-1, "XX")
        self.assertFalse(ncs_chain_info, "   Fail: ncs-chains returns %s, should be False" %ncs_chain_info)

        # should return False for insulin
        #global imol_insulin
        ncs_chain_info = coot.ncs_chain_differences_py(imol_insulin(), "A")
        self.assertFalse(ncs_chain_info, "   Fail: ncs-chains for insulin returns %s, should be False" %ncs_chain_info)

        # a normal case
        imol_rnase = coot.read_pdb(coot_testing_utils.rnase_pdb())
        coot.make_ncs_ghosts_maybe(imol_rnase)
        ncs_chain_info = coot.ncs_chain_differences_py(imol_rnase, "A")
        self.assertTrue(ncs_chain_info, "   Fail: ncs-chain-differences returns False")
        self.assertEqual(len(ncs_chain_info), 3,
                             """   Fail on length: length ncs-chain-differences should be 3 is %s\n
                                     ncs-chain-differences returns %s""" %(len(ncs_chain_info), ncs_chain_info))


# Phil spotted this bug (it wouldn't refine across the residue 3->4
# peptide bond - because out out of order residues
#
    def test04_0(self):
        """NCS Residue Range copy"""

        # ls is a list of numbers. Is it in ascending order? Return True or False
        def ascending_order_qm(ls):
            asc = sorted(ls)
            if (ls == asc):
                return True
            else:
                return False

        # prepare the input
        imol = coot.read_pdb(coot_testing_utils.rnase_pdb())
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol), "fail to read %s" %coot_testing_utils.rnase_pdb())
        for r in range(1, 4):
            coot.delete_residue(imol, "B", r, "")
        # make ghosts
        coot.make_ncs_ghosts_maybe(imol)
        # evaluate the function to be tested
        coot.copy_residue_range_from_ncs_master_to_others(imol, "A", 1, 6)
        # check the result
        chain_id = "B"
        n_residues = coot.chain_n_residues(chain_id, imol)
        seqnum_order = [coot.seqnum_from_serial_number(imol, chain_id, serial_number) for serial_number in range(n_residues)]

        # return a boolean value
        self.assertTrue(ascending_order_qm(seqnum_order), "fail with seqnum_order %s not ascending" %seqnum_order)

        # Now we can test a mutation.
        #
        # check that the mutate function returns success.
        #
        # check that the new residue type is indeed a TRP
        #
        # make the NCS copy
        #
        # check that the NCS target residue has had it's type
        # updated to TRP.
        #
        mutate_success = coot.mutate(imol, "A", 2, "", "TRP")
        self.assertTrue(mutate_success == 1, "Mutate fails.")

        rname = coot.residue_name(imol, "A", 2, "")
        self.assertTrue(rname == "TRP", "Mutate fails - master not a TRP. %s" %rname)

        coot.copy_residue_range_from_ncs_master_to_others(imol, "A", 1, 3)
        rname = coot.residue_name(imol, "B", 2, "")
        self.assertTrue(rname == "TRP", "Mutate fails - peer not a TRP (%s)" %rname)


# Perhaps combine this with previous test, its tests the same function.
#
    def test05_0(self):
        """NCS Residue Range edit to all chains"""

        imol = coot_testing_utils.unittest_pdb("pdb1t6q.ent")
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol))

        coot.mutate(imol, "A", 50, "", "ASP")
        coot_ncs.skip_to_next_ncs_chain("forward")     # generate the ghosts

        coot.copy_residue_range_from_ncs_master_to_others(imol, "A", 50, 50)

        # did it apply?
        result = []
        for chain_id_peer in ["B", "C"]:
            resname = coot.residue_name(imol, chain_id_peer, 50, "")
            if (resname == "ASP"):
                result.append(True)
            else:
                result.append(False)
        print("result:", result)
        self.assertTrue(all(result))


# This excercises a failure reported by Engin Ozkan 20081209.  Oh
# D'oh, I'd hard-coded "A" as the master chain id into
# manual-ncs-ghosts, I should have used the beginning of the list of
# chain-ids instead.
#
    def test06_0(self):
        """Manual NCS ghosts generates correct NCS chain ids"""

        imol = coot_testing_utils.unittest_pdb("pdb1hvv.ent")

        coot.set_draw_ncs_ghosts(imol, 1)
        coot.ncs_control_change_ncs_master_to_chain_id(imol, "B")
        coot.make_ncs_ghosts_maybe(imol)
        ncs_ghost_chains_1 = coot_utils.ncs_chain_ids(imol)
        coot_ncs.manual_ncs_ghosts(imol, 220, 230, ["B", "A", "C", "D"])
        ncs_ghost_chains_2 = coot_utils.ncs_chain_ids(imol)

        print("   NCS ghost chain IDs pre:  ", ncs_ghost_chains_1)
        print("   NCS ghost chain IDs post: ", ncs_ghost_chains_2)

        self.assertEqual(ncs_ghost_chains_1, [["B", "A", "C", "D"]])
        self.assertEqual(ncs_ghost_chains_2, [["B", "A", "C", "D"]])


    def test07_0(self):
        """NCS maps overwrite existing maps"""

        # first close all the maps that have "NCS found" in the name:
        for imol in coot_utils.molecule_number_list():
            if ("NCS found" in coot.molecule_name(imol)):
                coot.close_molecule(imol)

        imol = coot_testing_utils.unittest_pdb("pdb1hvv.ent")
        imol_map = coot.make_and_draw_map(os.path.join(coot_testing_utils.unittest_data_dir, "1hvv_sigmaa.mtz"),
                                          "2FOFCWT", "PH2FOFCWT", "", 0, 0)

        coot.make_dynamically_transformed_ncs_maps(imol, imol_map, 0)
        coot.make_dynamically_transformed_ncs_maps(imol, imol_map, 0)
        coot.make_dynamically_transformed_ncs_maps(imol, imol_map, 1)

        result_list = []
        molecule_names = list(map(coot.molecule_name, coot_utils.molecule_number_list()))
        print("BL DEBUG:: molecule_names", molecule_names)
        for chain_id in ["B", "C", "D"]:
            test_name = "Map " + \
                        str(imol_map) + " " + \
                        "NCS found from matching Chain " + \
                        chain_id + \
                        " onto Chain A"

            n_matchers = molecule_names.count(test_name)
            # print("DEBUG:: n_matchers", n_matchers)
            self.assertTrue(n_matchers >= 2, "  Failed to find matching NCS chain %s" %chain_id)



#!/usr/bin/env python
"""Allows functions from coot_utils to be imported"""

# Copyright 2011, 2012 Kevin Keating
# 
# Licensed under the Educational Community License, Version 2.0 (the
# "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
# 
# http://www.osedu.org/licenses/ECL-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.

#"import coot_utils" results in an error, so this module is required to retrieve
#functions that are defined in coot_utils

import os, sys
from os.path import exists, join
from coot import *

use_gui_qm = False #coot_utils requires this variable to be defined

#search the Python path for coot_utils
for curpath in sys.path:
    abspath = join(curpath, "coot_utils.py")
    if exists(abspath):
        #when we find it, exec it
        
        #but first exec redefine_functions.py if it's in the same directory
        #redefine_functions.py renames func_py() to func(), which used to be done in coot_utils.py itself
        #new versions of coot_utils.py requires this renaming to be done before being exec'ed
        redefAbspath = join(curpath, "redefine_functions.py")
        if exists(redefAbspath):
            execfile(redefAbspath)
        
        execfile(abspath)
        break




import gi
gi.require_version('Gtk', '4.0')
from gi.repository import Gtk
import coot
import coot_gui_api
import coot_utils
import coot_gui

def chain_refine(imol, ch_id):
    residue_list = coot_utils.residues_in_chain(imol, ch_id)
    coot.refine_residues_py(imol, residue_list)

# this might be a coot_gui function now
def coot_menubar_menu(s):
   menu = Gtk.Menu()
   menuitem = Gtk.MenuItem(s)
   menuitem.set_submenu(menu)
   main_menubar = coot_gui_api.main_menubar()
   main_menubar.append(menuitem)
   menuitem.show()
   return menu

def add_module_refine():

    menu_refine = coot_menubar_menu("Refine")

    def all_atom_refine_func():
        active_atom = coot.active_residue_py()
        if active_atom:
            imol = active_atom[0]
            residue_list = coot_utils.all_residues(imol)
            coot.refine_residues_py(imol, residue_list)

    def chain_refine_func():
        active_atom = coot.active_residue_py()
        if active_atom:
            imol = active_atom[0]
            ch_id = active_atom[1]
            chain_refine(imol, ch_id)

    def refine_active_fragment_func():
        active_atom = coot.active_residue_py()
        if active_atom:
            imol = active_atom[0]
            ch_id = active_atom[1]
            residue_list = coot.linked_residues_py(res_spec, imol, 1.7)
            coot.refine_residues_py(imol, residue_list)
        
    coot_gui.add_simple_coot_menu_menuitem(menu_refine, "All-atom Refine", lambda arg: all_atom_refine_func())
    coot_gui.add_simple_coot_menu_menuitem(menu_refine, "Chain Refine",    lambda arg: chain_refine_func())
    coot_gui.add_simple_coot_menu_menuitem(menu_refine, "Refine Fragment", lambda arg: chain_refine_func())

    coot_gui.add_simple_coot_menu_menuitem(menu_refine, "Contact Dots On",  lambda arg: coot.set_do_coot_probe_dots_during_refine(1))
    coot_gui.add_simple_coot_menu_menuitem(menu_refine, "Contact Dots Off", lambda arg: coot.set_do_coot_probe_dots_during_refine(0))

    coot_gui.add_simple_coot_menu_menuitem(menu_refine, "Rotamer Markup On",   lambda arg: coot.set_show_intermediate_atoms_rota_markup(1))
    coot_gui.add_simple_coot_menu_menuitem(menu_refine, "Rotamer Markup Off",  lambda arg: coot.set_show_intermediate_atoms_rota_markup(0))

    coot_gui.add_simple_coot_menu_menuitem(menu_refine, "Refine Rotamers On",  lambda arg: coot.set_refine_rotamers(1))
    coot_gui.add_simple_coot_menu_menuitem(menu_refine, "Refine Rotamers Off", lambda arg: coot.set_refine_rotamers(0))
